/* -*- C++ -*- 
 * mu_state.h
 * @(#) header for routines related to states in the verifier
 *
 * Copyright (C) 1992 - 1999 by the Board of Trustees of              
 * Leland Stanford Junior University.
 *
 * License to use, copy, modify, sell and/or distribute this software
 * and its documentation any purpose is hereby granted without royalty,
 * subject to the following terms and conditions:
 *
 * 1.  The above copyright notice and this permission notice must
 * appear in all copies of the software and related documentation.
 *
 * 2.  The name of Stanford University may not be used in advertising or
 * publicity pertaining to distribution of the software without the
 * specific, prior written permission of Stanford.
 *
 * 3.  This software may not be called "Murphi" if it has been modified
 * in any way, without the specific prior written permission of David L.
 * Dill.
 *
 * 4.  THE SOFTWARE IS PROVIDED "AS-IS" AND STANFORD MAKES NO
 * REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED, BY WAY OF EXAMPLE,
 * BUT NOT LIMITATION.  STANFORD MAKES NO REPRESENTATIONS OR WARRANTIES
 * OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE OR THAT THE
 * USE OF THE SOFTWARE WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS
 * TRADEMARKS OR OTHER RIGHTS. STANFORD SHALL NOT BE LIABLE FOR ANY
 * LIABILITY OR DAMAGES WITH RESPECT TO ANY CLAIM BY LICENSEE OR ANY
 * THIRD PARTY ON ACCOUNT OF, OR ARISING FROM THE LICENSE, OR ANY
 * SUBLICENSE OR USE OF THE SOFTWARE OR ANY SERVICE OR SUPPORT.
 *
 * LICENSEE shall indemnify, hold harmless and defend STANFORD and its
 * trustees, officers, employees, students and agents against any and all
 * claims arising out of the exercise of any rights under this Agreement,
 * including, without limiting the generality of the foregoing, against
 * any damages, losses or liabilities whatsoever with respect to death or
 * injury to person or damage to property arising from or out of the
 * possession, use, or operation of Software or Licensed Program(s) by
 * LICENSEE or its customers.
 *
 * Read the file "license" distributed with these sources, or call
 * Murphi with the -l switch for additional information.
 *
 *
 */

/* 
 * Original Author: Ralph Melton
 * Extracted from mu_epilog.inc and mu_prolog.inc
 * by C. Norris Ip
 * Created: 21 April 93
 *
 */ 

/* 
 * Cached Murphi Release 4.1
 * Author: G. Della Penna
 * Released: 30 Jan 2003
 * Notes: Several methods were modified or added to support
 *        the disk management routines. See README.cmurphi for more
 *        information.
 *       
 */

/* 
 * Cached Murphi Release 4.2
 * Author: G. Della Penna
 * Released: 01 May 2003
 * Notes: Cached Murphi now supports (finite precision) real numbers.
 *       
 */


#ifndef _STATE_
#define _STATE_

/****************************************
  There are three different declarations:
  1) state
  2) dynBitVec
 ****************************************/

/****************************************
  The record for a single state.
  require : BITS_IN_WORLD in parameter file
 ****************************************/

/* BITS_IN_WORLD gets defined by the generated code. */
/* The extra addition is there so that we round up to the greater block. */

/****************************************
  Bit vector - copied straight from Andreas. 
 ****************************************/
class dynBitVec
{
  // data
  unsigned long numBits;
  unsigned char* v;
  
  // Inquiries
  inline unsigned int Index( unsigned long index ) { return index / 8; }
  inline unsigned int Shift( unsigned long index ) { return index % 8; }
  
public:
  // initializer
  dynBitVec( unsigned long nBits );
  // destructor
  virtual ~dynBitVec();
  
  // interface
  inline int NumBits( void ) { return numBits; }
  inline int NumBytes( void ) { return 1 + (numBits - 1) / 8; }
  inline void clear( unsigned long i ) { v[ Index(i) ] &= ~(1 << Shift(i)); }
  inline void set( unsigned long i ) { v[ Index(i) ] |=  (1 << Shift(i)); }
  inline int get( unsigned long i ) { return (v[ Index(i) ] >> Shift(i)) & 1; }

  inline void clearall()   {memset( v, 0, NumBytes() );}
  inline void setall()   {memset( v, 1, NumBytes() );}
};

class statelist
{
  state * s;
  statelist * next;
public:
  statelist(state * s, statelist * next) 
  : s(s), next(next) {};
};

/****************************************
  1) 1 Dec 93 Norris Ip: 
  check -sym option when checking was_present()
  add Normalize() declaration in class state
  add friend StateCmp to class state
  2) 24 Feb 94 Norris Ip:
  added -debugsym option to run two hash tables in parallel
  for debugging purpose
  3) 8 March 94 Norris Ip:
  merge with the latest rel2.6
****************************************/

/********************
  $Log: mu_state.h,v $
  
  Revision 2.0  2003/01/10 17:06:12  giuseppe 
  disk extension - final release

  Revision 1.5  2001/10/28 21:50:00  giuseppe 
  disk algorithm support complete - start testing

  Revision 1.4  2001/10/01 18:41:14  giuseppe 
  start adding support for disk algorithm
  
  Revision 1.3  2001/06/11 16:39:11  giuseppe 
  cache extension - final release  
  
  Revision 1.2  2001/02/28 20:32:12  enrico
  modified simple_was_present

  Revision 1.1  2001/02/02 08:44:28  enrico
  Initial revision

*******************

  Revision 1.3  1999/01/29 08:28:09  uli
  efficiency improvements for security protocols

  Revision 1.2  1999/01/29 07:49:11  uli
  bugfixes

  Revision 1.4  1996/08/07 18:54:33  ip
  last bug fix on NextRule/SetNextEnabledRule has a bug; fixed this turn

  Revision 1.3  1996/08/07 01:00:18  ip
  Fixed bug on what_rule setting during guard evaluation; otherwise, bad diagnoistic message on undefine error on guard

  Revision 1.2  1996/08/07 00:15:26  ip
  fixed while code generation bug

  Revision 1.1  1996/08/07 00:14:46  ip
  Initial revision

********************/

#endif
